from onegov.wtfs.layouts.daily_list import DailyListBoxesAndFormsLayout
from onegov.wtfs.layouts.daily_list import DailyListBoxesLayout
from onegov.wtfs.layouts.daily_list import DailyListLayout
from onegov.wtfs.layouts.default import DefaultLayout
from onegov.wtfs.layouts.invoice import InvoiceLayout
from onegov.wtfs.layouts.mail import MailLayout
from onegov.wtfs.layouts.municipality import AddMunicipalityLayout
from onegov.wtfs.layouts.municipality import DeleteMunicipalityDatesLayout
from onegov.wtfs.layouts.municipality import EditMunicipalityLayout
from onegov.wtfs.layouts.municipality import ImportMunicipalityDataLayout
from onegov.wtfs.layouts.municipality import MunicipalitiesLayout
from onegov.wtfs.layouts.municipality import MunicipalityLayout
from onegov.wtfs.layouts.notification import AddNotificationLayout
from onegov.wtfs.layouts.notification import EditNotificationLayout
from onegov.wtfs.layouts.notification import NotificationLayout
from onegov.wtfs.layouts.notification import NotificationsLayout
from onegov.wtfs.layouts.payment_type import PaymentTypesLayout
from onegov.wtfs.layouts.report import ReportBoxesAndFormsByDeliveryLayout
from onegov.wtfs.layouts.report import ReportBoxesAndFormsLayout
from onegov.wtfs.layouts.report import ReportBoxesLayout
from onegov.wtfs.layouts.report import ReportFormsByMunicipalityLayout
from onegov.wtfs.layouts.report import ReportLayout
from onegov.wtfs.layouts.scan_job import AddScanJobLayout
from onegov.wtfs.layouts.scan_job import DeliveryNoteLayout
from onegov.wtfs.layouts.scan_job import EditScanJobLayout
from onegov.wtfs.layouts.scan_job import ScanJobLayout
from onegov.wtfs.layouts.scan_job import ScanJobsLayout
from onegov.wtfs.layouts.user import AddUserLayout
from onegov.wtfs.layouts.user import EditUserLayout
from onegov.wtfs.layouts.user import UserLayout
from onegov.wtfs.layouts.user import UsersLayout
from onegov.wtfs.layouts.user_manual import EditUserManualLayout
from onegov.wtfs.layouts.user_manual import UserManualLayout


__all__ = (
    'AddMunicipalityLayout',
    'AddNotificationLayout',
    'AddScanJobLayout',
    'AddUserLayout',
    'DailyListBoxesAndFormsLayout',
    'DailyListBoxesLayout',
    'DailyListLayout',
    'DefaultLayout',
    'DeleteMunicipalityDatesLayout',
    'DeliveryNoteLayout',
    'EditMunicipalityLayout',
    'EditNotificationLayout',
    'EditScanJobLayout',
    'EditUserLayout',
    'EditUserManualLayout',
    'ImportMunicipalityDataLayout',
    'InvoiceLayout',
    'MailLayout',
    'MunicipalitiesLayout',
    'MunicipalityLayout',
    'NotificationLayout',
    'NotificationsLayout',
    'PaymentTypesLayout',
    'ReportBoxesAndFormsByDeliveryLayout',
    'ReportBoxesAndFormsLayout',
    'ReportBoxesLayout',
    'ReportFormsByMunicipalityLayout',
    'ReportLayout',
    'ScanJobLayout',
    'ScanJobsLayout',
    'UserLayout',
    'UserManualLayout',
    'UsersLayout',
)
